//--------------------------------------------------------------
// File     : stm32_ub_cs43l22.c
// Datum    : 09.06.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : STM32_UB_I2C1, SPI, DMA, MISC
// Funktion : CS43L22 (2Kanal Audio-DAC)
//            zum abspielen von WAV-Files aus dem Flash
//            [16bit, PCM, Mono/Stereo, 8kHz bis 48kHz]
//
// Hinweis  : benutzt I2C1 und I2S3
// 
//            I2C =>  PB9  = SDA   [SlaveAdr=0x94]
//                    PB6  = SCL
//            I2S =>  PA4  = WS
//                    PC7  = MCLK
//                    PC10 = SCK
//                    PC12 = SD
//            Reset = PD4
//
//      zum konvertieren von WAV-Files kann das PC-Programm :
//      "FileConverter_UB.exe" benutzt werden
//            
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_cs43l22.h"




//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
CS43L22_STATUS_t cs43l22_status=CS43L22_NO_INIT;
WAV_t my_wav;
const uint8_t *wav_start;
static uint8_t wav_loop=0;


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
WAV_ERR_t P_CS43L22_CheckWAV(UB_WAV *wav);
uint32_t P_CS43L22_4Bytes(UB_WAV *wav, uint32_t start);
uint16_t P_CS43L22_2Bytes(UB_WAV *wav, uint32_t start);
void P_CS43L22_initWAV(void);
void P_CS43L22_PlayAgain(void);



//--------------------------------------------------------------
// Initialisierung vom CS43L22
// Return_wert :
//  -> ERROR   , wenn Initialisierung fehlgeschlagen
//  -> SUCCESS , wenn Initialisierung ok war 
//--------------------------------------------------------------
ErrorStatus UB_CS43L22_Init(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t check;

  if(cs43l22_status==CS43L22_NO_INIT) {

    // init der WAV-Struktur
    P_CS43L22_initWAV();

    // Clock der I2S einstellen
    UB_AUDIO_SetI2SClock();
    // Init und Check ob ok
    check=EVAL_AUDIO_Init(OUTPUT_DEVICE_HEADPHONE, 70, I2S_AudioFreq_48k);
    if(check==0) {
      ret_wert=SUCCESS;
      cs43l22_status=CS43L22_INIT_OK;
    }
    else {
      ret_wert=ERROR;
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// spielt WAV-File einmal ab (aus dem Flash)
// -> WAV muss mit &-Operator uebergeben werden
// Volume : [0...255]
// Return_wert :
//  ->   WAV_OK , Header war ok, WAV wird abgespielt
//  -> !=WAV_OK , Fehler im Header, WAV wird nicht abgespielt
//--------------------------------------------------------------
WAV_ERR_t UB_CS43L22_PlayWAVSingle(UB_WAV *wav, uint8_t Volume)
{
  WAV_ERR_t ret_wert=WAV_INIT_ERR;

  if((cs43l22_status==CS43L22_INIT_OK) || (cs43l22_status==CS43L22_STOP)) {
    // check vom WAV-Header
    ret_wert=P_CS43L22_CheckWAV(wav);
    if(ret_wert==WAV_OK) {
      cs43l22_status=CS43L22_PLAY;
      wav_loop=0; // single play
      wav_start=&wav->table[my_wav.data_start];
      if(my_wav.channels==2) {
        // stereo
        UB_AUDIO_InitNew(OUTPUT_DEVICE_HEADPHONE, Volume, my_wav.samplerate);
      }
      else {
        // mono
        UB_AUDIO_InitNew(OUTPUT_DEVICE_HEADPHONE, Volume, my_wav.samplerate/2);
      }
      EVAL_AUDIO_Play((uint16_t*)wav_start, my_wav.data_len);
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// spielt WAV-File in einer Endloosloop ab (aus dem Flash)
// -> WAV muss mit &-Operator uebergeben werden
// Volume : [0...255]
// Return_wert :
//  ->   WAV_OK , Header war ok, WAV wird abgespielt
//  -> !=WAV_OK , Fehler im Header, WAV wird nicht abgespielt
//--------------------------------------------------------------
WAV_ERR_t UB_CS43L22_PlayWAVLoop(UB_WAV *wav, uint8_t Volume)
{
  WAV_ERR_t ret_wert=WAV_INIT_ERR;

  if((cs43l22_status==CS43L22_INIT_OK) || (cs43l22_status==CS43L22_STOP)) {
    // check vom WAV-Header
    ret_wert=P_CS43L22_CheckWAV(wav);
    if(ret_wert==WAV_OK) {
      cs43l22_status=CS43L22_PLAY;
      wav_loop=1; // endless play
      wav_start=&wav->table[my_wav.data_start];
      if(my_wav.channels==2) {
        // stereo
        UB_AUDIO_InitNew(OUTPUT_DEVICE_HEADPHONE, Volume, my_wav.samplerate);
      }
      else {
        // mono
        UB_AUDIO_InitNew(OUTPUT_DEVICE_HEADPHONE, Volume, my_wav.samplerate/2);
      }
      EVAL_AUDIO_Play((uint16_t*)wav_start, my_wav.data_len);
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// Pausen-Funktion
//--------------------------------------------------------------
void UB_CS43L22_PauseWAV(void)
{
  if(cs43l22_status==CS43L22_PLAY) {
    cs43l22_status=CS43L22_PAUSE;
    EVAL_AUDIO_PauseResume(AUDIO_PAUSE); 
  }
}


//--------------------------------------------------------------
// Resume-Funktion
//--------------------------------------------------------------
void UB_CS43L22_ResumeWAV(void)
{
  if(cs43l22_status==CS43L22_PAUSE) {
    cs43l22_status=CS43L22_PLAY;
    EVAL_AUDIO_PauseResume(AUDIO_RESUME); 
  }
}


//--------------------------------------------------------------
// Stop-Funktion
//--------------------------------------------------------------
void UB_CS43L22_StopWAV(void)
{
  if((cs43l22_status==CS43L22_PLAY) || (cs43l22_status==CS43L22_PAUSE)) {
    cs43l22_status=CS43L22_STOP;
    EVAL_AUDIO_Stop(CODEC_PDWN_HW);  // CODEC_PDWN_SW => rauschen
  }
}


//--------------------------------------------------------------
// stellt Lautstrke ein
//--------------------------------------------------------------
void UB_CS43L22_SetVolume(uint8_t Volume)
{
  EVAL_AUDIO_VolumeCtl(Volume);
}


//--------------------------------------------------------------
// interne Funktion
// prft Header von WAV auf Plausibilitt
// erlaubt sind nur 16bit, PCM-Files (Mono oder Stereo)
// mit 8kHz bis 48kHz
//
// Return_wert :
//  ->    WAV_OK , wenn Header ok war
//  -> != WAV_OK , wenn Fehler im Header
//--------------------------------------------------------------
WAV_ERR_t P_CS43L22_CheckWAV(UB_WAV *wav)
{
  const uint8_t *wert;
  uint32_t offset=0;

  // Reset der WAV-Struktur
  P_CS43L22_initWAV();

  wert=&wav->table[0];

  // Filesize bei WAV muss > 50 Bytes sein
  if(wav->size<50) return(WAV_SIZE_ERR);

  // check auf "RIFF-ID"
  if((wert[0]!='R') || (wert[1]!='I') || (wert[2]!='F') || (wert[3]!='F')) return(WAV_ID_ERR);

  // file lnge auslesen (egal)
  my_wav.file_len=P_CS43L22_4Bytes(wav,4);

  // check auf "WAV-ID"
  if((wert[8]!='W') || (wert[9]!='A') || (wert[10]!='V') || (wert[11]!='E')) return(WAV_ID2_ERR);

  // check auf "fmt-ID"
  if((wert[12]!='f') || (wert[13]!='m') || (wert[14]!='t') || (wert[15]!=' ')) return(WAV_ID3_ERR);

  // fmt laenge auslesen (16d oder 18d)
  my_wav.fmt_len=P_CS43L22_4Bytes(wav,16);
  if((my_wav.fmt_len!=16) && (my_wav.fmt_len!=18)) return(WAV_FMT_LEN_ERR);
  if(my_wav.fmt_len==16) offset=0;
  if(my_wav.fmt_len==18) offset=14;

  // File-Typ auslesen (0x01=PCM)
  my_wav.typ=P_CS43L22_2Bytes(wav,20);
  if(my_wav.typ!=0x01) return(WAV_TYP_ERR);

  // Anzahl Kanle auslesen (0x01=Mono, 0x02=Stereo)
  my_wav.channels=P_CS43L22_2Bytes(wav,22);
  if((my_wav.channels<1) || (my_wav.channels>2)) return(WAV_CH_ERR);

  // Sample Rate auslesen (8kHz bis 48kHz)
  my_wav.samplerate=P_CS43L22_4Bytes(wav,24);
  if((my_wav.samplerate<8000) || (my_wav.samplerate>48000))  return(WAV_FRQ_ERR);

  // Byterate auslesen (in Bytes/sec)
  my_wav.byterate=P_CS43L22_4Bytes(wav,28);

  // Framesize auslesen (1,2,4)
  my_wav.framesize=P_CS43L22_2Bytes(wav,32);
  if((my_wav.framesize!=1) && (my_wav.framesize!=2) && (my_wav.framesize!=4)) return(WAVE_FRAME_ERR);

  // Bits pro Sample auslesen (16d)
  my_wav.bitpersample=P_CS43L22_2Bytes(wav,34);
  if(my_wav.bitpersample!=16) return(WAVE_BPS_ERR);

  // check auf "DATA-ID"
  if((wert[36+offset]!='d') || (wert[37+offset]!='a') || (wert[38+offset]!='t') || (wert[39+offset]!='a')) return(WAVE_ID4_ERR);

  // Data Lnge auslesen
  my_wav.data_len=P_CS43L22_4Bytes(wav,40+offset);

  // Data-Start Adresse setzen
  my_wav.data_start=44+offset;

  return(WAV_OK);
}


//--------------------------------------------------------------
// interne Funktion
// DWORD (4Bytes) vom Puffer ab start auslesen
//--------------------------------------------------------------
uint32_t P_CS43L22_4Bytes(UB_WAV *wav, uint32_t start)
{
  uint32_t ret_wert=0;
  const uint8_t *wert;

  wert=&wav->table[0];

  ret_wert|=(wert[start]);
  ret_wert|=(wert[start+1]<<8);
  ret_wert|=(wert[start+2]<<16);
  ret_wert|=(wert[start+3]<<24);

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// WORD (2Bytes) vom Puffer ab start auslesen
//--------------------------------------------------------------
uint16_t P_CS43L22_2Bytes(UB_WAV *wav, uint32_t start)
{
  uint16_t ret_wert=0;
  const uint8_t *wert;

  wert=&wav->table[0];

  ret_wert|=(wert[start]);
  ret_wert|=(wert[start+1]<<8);

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// init der WAV-Parameter
//--------------------------------------------------------------
void P_CS43L22_initWAV(void)
{
  my_wav.file_len=0;
  my_wav.fmt_len=0;
  my_wav.typ=0;
  my_wav.channels=0;
  my_wav.samplerate=0;
  my_wav.byterate=0;
  my_wav.framesize=0;
  my_wav.bitpersample=0;
  my_wav.data_len=0;
  my_wav.data_start=0;
}


//--------------------------------------------------------------
// interne Funktion
// spielt aktuelles WAV nochmal ab
//--------------------------------------------------------------
void P_CS43L22_PlayAgain(void)
{
  EVAL_AUDIO_Play((uint16_t*)wav_start, my_wav.data_len);
}

//--------------------------------------------------------------
uint16_t EVAL_AUDIO_GetSampleCallBack(void)
{
  return 0; // nc
}


//--------------------------------------------------------------
// wird aufgerufen, wenn WAV-File komplett abgespielt wurde
//--------------------------------------------------------------
void EVAL_AUDIO_TransferComplete_CallBack(uint32_t pBuffer, uint32_t Size)
{
#ifdef AUDIO_MAL_MODE_NORMAL
  if(wav_loop==1) {
    P_CS43L22_PlayAgain();
  }
  else {
    UB_CS43L22_StopWAV();
  }
#else

   

#endif
}



//--------------------------------------------------------------
void EVAL_AUDIO_HalfTransfer_CallBack(uint32_t pBuffer, uint32_t Size)
{  
#ifdef AUDIO_MAL_MODE_CIRCULAR
  
#else
  
#endif
}


//--------------------------------------------------------------
void EVAL_AUDIO_Error_CallBack(void* pData)
{
  // nc
}




